@@ -10,13 +10,17 @@
                 names(Call), nomatch=0) 
   if (indx[1] ==0) stop("A formula argument is required");
   temp <- Call[c(1,indx)]  # only keep the arguments we wanted
-  temp[[1]] <- as.name('model.frame')  # change the function called
+  temp[[1L]] <- quote(stats::model.frame)
   
-  special <- c("baseline", "frailtyprior", "truncation_time", "subject.num")
-  temp$formula <- if(missing(data)) terms(formula, special)
-  else              terms(formula, special, data=data)
-  if (is.R()) m <- eval(temp, parent.frame())
-  else        m <- eval(temp, sys.parent())
+  special <- c("baseline", "frailtyprior", "truncation_time", "subject.num", "bspline")
+  temp$formula <- if (missing(data)) 
+    terms(formula, special)
+  else terms(formula, special, data = data)
+  
+  if (is.R()) 
+    m <- eval(temp, parent.frame())
+  else m <- eval(temp, sys.parent())
+  Terms <- attr(m, 'terms')
   
   if(any(names(m)=="(truncation_time)")){
     truncation_time = m[,"(truncation_time)"]
@@ -30,28 +34,44 @@
     subject.num = NULL
   }
   
-  Terms <- attr(m, 'terms')
   Y <- model.extract(m, "response")
   if (!inherits(Y, "Surv")) stop("Response must be a survival object")
   
   baseline0 <- attr(Terms, "specials")$baseline
   frailtyprior0<- attr(Terms, "specials")$frailtyprior
-  dropx <- NULL
+  bspline0<- attr(Terms, "specials")$bspline
+  
   if (length(frailtyprior0)) {
     temp <- survival::untangle.specials(Terms, 'frailtyprior', 1)
-    dropx <- c(dropx, temp$terms)
+    dropfrail <- c(temp$terms)
     frail.terms <- m[[temp$vars]]
   }else{
+    dropfrail <- NULL
     frail.terms <- NULL;
   }
   if (length(baseline0)) {
     temp <- survival::untangle.specials(Terms, 'baseline', 1)
-    dropx <- c(dropx, temp$terms)
+    dropXtf <- c(temp$terms)
     Xtf <- m[[temp$vars]]
   }else{
-    Xtf <- NULL;
+    dropXtf <- NULL
+    Xtf <- NULL
+  }
+  if (length(bspline0)) {
+    temp <- survival::untangle.specials(Terms, 'bspline', 1)
+    #dropx <- c(dropx, temp$terms);
+    X.bs = NULL;
+    n.bs = rep(0, length(temp$vars));
+    for(ii in 1:length(temp$vars)){
+      X.bs = cbind(X.bs, m[[temp$vars[ii]]]);
+      n.bs[ii] = ncol(m[[temp$vars[ii]]]); 
+    }
+  }else{
+    X.bs <- NULL;
+    n.bs <- NULL;
   }
   
+  dropx <- c(dropfrail, dropXtf)
   if (length(dropx)) {
     newTerms <- Terms[-dropx]
     # R (version 2.7.1) adds intercept=T anytime you drop something
@@ -82,6 +102,7 @@
   xdrop <- Xatt$assign %in% adrop  #columns to drop (always the intercept)
   X <- X[, !xdrop, drop=FALSE]
   attr(X, "assign") <- Xatt$assign[!xdrop]
+  
   n <- nrow(X)
   p <- ncol(X)
   if(p==0){
@@ -277,6 +298,7 @@
   
   #### Save to a list
   output <- list(modelname=model.name,
+                 terms=m,
                  dist = dist,
                  coefficients=coeff,
                  call=Call,
@@ -384,36 +406,86 @@
   invisible(x)
 }
 
-"plot.SuperSurvRegBayes" <- function (x, xpred=NULL, tgrid=NULL, CI=0.95, PLOT=FALSE, ...) {
-  if(is(x,"SuperSurvRegBayes")){
-    if(is.null(tgrid)) tgrid = seq(0.01, max(x$Surv[,2], na.rm=T), length.out=200);
-    if(is.null(xpred)) {
-      stop("please specify xpred")
+"plot.SuperSurvRegBayes" <- function(x, xnewdata, tgrid=NULL, CI=0.95, PLOT=TRUE, ...) {
+  if(is.null(tgrid)) tgrid = seq(0.01, max(x$Surv[,1], na.rm=T), length.out=200);
+  if(missing(xnewdata)) {
+    stop("please specify xnewdata")
+  }else{
+    rnames = row.names(xnewdata)
+    m = x$terms
+    Terms = attr(m, 'terms')
+    baseline0 <- attr(Terms, "specials")$baseline
+    frailtyprior0<- attr(Terms, "specials")$frailtyprior
+    dropx <- NULL
+    if (length(frailtyprior0)) {
+      temp <- survival::untangle.specials(Terms, 'frailtyprior', 1)
+      dropx <- c(dropx, temp$terms)
+      frail.terms <- m[[temp$vars]]
     }else{
-      if(is.vector(xpred)) xpred=matrix(xpred, nrow=1);
-      if(ncol(xpred)!=x$p) stop("please make sure the number of columns matches!");
-    };
-    X.center = attributes(x$X.scaled)$`scaled:center`;
-    X.scale = attributes(x$X.scaled)$`scaled:scale`;
-    xpred = cbind(xpred);
-    nxpred = nrow(xpred);
-    for(i in 1:nxpred) xpred[i,] = (xpred[i,]-X.center)/X.scale;
-    distcode = switch(x$dist, loglogistic=1, lognormal=2, 3);
-    estimates <- .Call("PHPOAFT_BP_plots", tgrid, xpred, x$theta.scaled, 
-                       x$beta_h.scaled, x$beta_o.scaled, x$beta_q.scaled, x$weight, CI, 
-                       dist_=distcode, PACKAGE = "spBayesSurv");
-    if(PLOT){
-      par(cex=1.5,mar=c(4.1,4.1,1,1),cex.lab=1.4,cex.axis=1.1)
-      plot(tgrid, estimates$Shat[,1], "l", lwd=3, xlab="time", ylab="survival", main=paste(i));
-      for(i in 1:nxpred){
-        polygon(x=c(rev(tgrid),tgrid),
-                y=c(rev(estimates$Shatlow[,i]),estimates$Shatup[,i]),
-                border=NA,col="lightgray");
-      }
-      for(i in 1:nxpred){
-        lines(tgrid, estimates$Shat[,i], lty=3, lwd=3, col=1);
-      }
+      frail.terms <- NULL;
     }
+    if (length(baseline0)) {
+      temp <- survival::untangle.specials(Terms, 'baseline', 1)
+      dropx <- c(dropx, temp$terms)
+      Xtf <- m[[temp$vars]]
+    }else{
+      Xtf <- NULL;
+    }
+    if (length(dropx)) {
+      newTerms <- Terms[-dropx]
+      # R (version 2.7.1) adds intercept=T anytime you drop something
+      if (is.R()) attr(newTerms, 'intercept') <- attr(Terms, 'intercept')
+    } else  newTerms <- Terms
+    newTerms <- delete.response(newTerms)
+    mnew <- model.frame(newTerms, xnewdata, na.action = na.omit, xlev = .getXlevels(newTerms, m))
+    Xnew <- model.matrix(newTerms, mnew);
+    if (is.R()) {
+      assign <- lapply(survival::attrassign(Xnew, newTerms)[-1], function(x) x-1)
+      xlevels <- .getXlevels(newTerms, mnew)
+      contr.save <- attr(Xnew, 'contrasts')
+    }else {
+      assign <- lapply(attr(Xnew, 'assign')[-1], function(x) x -1)
+      xvars <- as.character(attr(newTerms, 'variables'))
+      xvars <- xvars[-attr(newTerms, 'response')]
+      if (length(xvars) >0) {
+        xlevels <- lapply(mnew[xvars], levels)
+        xlevels <- xlevels[!unlist(lapply(xlevels, is.null))]
+        if(length(xlevels) == 0)
+          xlevels <- NULL
+      } else xlevels <- NULL
+      contr.save <- attr(Xnew, 'contrasts')
+    }
+    # drop the intercept after the fact, and also drop baseline if necessary
+    adrop <- 0  #levels of "assign" to be dropped; 0= intercept
+    Xatt <- attributes(Xnew) 
+    xdrop <- Xatt$assign %in% adrop  #columns to drop (always the intercept)
+    Xnew <- Xnew[, !xdrop, drop=FALSE]
+    attr(Xnew, "assign") <- Xatt$assign[!xdrop]
+    xpred = Xnew
+    if(ncol(xpred)!=x$p) stop("please make sure the number of columns matches!");
+  }
+  X.center = attributes(x$X.scaled)$`scaled:center`;
+  X.scale = attributes(x$X.scaled)$`scaled:scale`;
+  xpred = cbind(xpred);
+  nxpred = nrow(xpred);
+  for(i in 1:nxpred) xpred[i,] = (xpred[i,]-X.center)/X.scale;
+  distcode = switch(x$dist, loglogistic=1, lognormal=2, 3);
+  estimates <- .Call("PHPOAFT_BP_plots", tgrid, xpred, x$theta.scaled, 
+                     x$beta_h.scaled, x$beta_o.scaled, x$beta_q.scaled, x$weight, CI, 
+                     dist_=distcode, PACKAGE = "spBayesSurv");
+  if(PLOT){
+    par(cex=1.5,mar=c(4.1,4.1,1,1),cex.lab=1.4,cex.axis=1.1)
+    plot(tgrid, estimates$Shat[,1], "l", lwd=3, xlab="time", ylab="survival", 
+         xlim=c(0, max(tgrid)), ylim=c(0,1));
+    for(i in 1:nxpred){
+      polygon(x=c(rev(tgrid),tgrid),
+              y=c(rev(estimates$Shatlow[,i]),estimates$Shatup[,i]),
+              border=NA,col="lightgray");
+    }
+    for(i in 1:nxpred){
+      lines(tgrid, estimates$Shat[,i], lty=i, lwd=3, col=i);
+    }
+    legend("topright", rnames, col = 1:nxpred, lty=1:nxpred, ...)
   }
   estimates$tgrid=tgrid;
   invisible(estimates)
